#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

__author__    = "Ira W. Snyder (devel@irasnyder.com)"
__copyright__ = "Copyright (c) 2006, Ira W. Snyder (devel@irasnyder.com)"
__license__   = "GNU GPL v2 (or, at your option, any later version)"

# This module holds all of the common functions used throughout the rarslave project.

import os
import re
import logging

import rsutil.globals
import rsutil.par2parser

# Global constants
(SUCCESS, ECHECK, EEXTRACT, EDELETE, ECREATE, EDETECT, EPARSE) = range(7)

def find_matches (regex, li, ignorecase=True):

	if ignorecase:
		cregex = re.compile (regex, re.IGNORECASE)
	else:
		cregex = re.compile (regex)

	return [e for e in li if cregex.match (e)]

def has_a_match (regex, li, ignorecase=True):

	if ignorecase:
		cregex = re.compile (regex, re.IGNORECASE)
	else:
		cregex = re.compile (regex)

	for e in li:
		if cregex.match (e):
			return True

	return False

def no_duplicates (li):
	"""Removes all duplicates from a list"""
	return list(set(li))

def run_command (cmd, indir=None):
	# Runs the specified command-line in the directory given (or, in the current directory
	# if none is given). It returns the status code given by the application.

	pwd = os.getcwd ()

	if indir != None:
		assert os.path.isdir (indir) # MUST be a directory!
		os.chdir (indir)

	ret = os.system (cmd)
	os.chdir (pwd)
	return ret

def full_abspath (p):
	return os.path.abspath (os.path.expanduser (p))

def find_par2_files (files):
	"""Find all par2 files in the list $files"""

	PAR2_REGEX = config_get_value ('regular expressions', 'par2_regex')
	regex = re.compile (PAR2_REGEX, re.IGNORECASE)
	return [f for f in files if regex.match (f)]

def find_name_matches (dir, basename):
	"""Finds files which are likely to be part of the set corresponding
	   to $name in the directory $dir"""

	assert os.path.isdir (dir)

	ename = re.escape (basename)
	regex = re.compile ('^%s.*$' % (ename, ))

	return [f for f in os.listdir (dir) if regex.match (f)]

def parse_all_par2 (dir, p2head, p2files):
	"""Searches though p2files and tries to parse at least one of them"""
	done = False
	files = []

	for f in p2files:

		# Exit early if we've found a good file
		if done:
			break

		try:
			files = rsutil.par2parser.get_protected_files (dir, f)
			done = True
		except (EnvironmentError, OSError, OverflowError):
			logging.warning ('Corrupt PAR2 file: %s' % f)

	# Now that we're out of the loop, check if we really finished
	if not done:
		logging.critical ('All PAR2 files corrupt for: %s' % p2head)

	# Return whatever we've got, empty or not
	return files

def get_basename (name):
	"""Strips most kinds of endings from a filename"""

	regex = config_get_value ('regular expressions', 'basename_regex')
	r = re.compile (regex, re.IGNORECASE)
	done = False

	while not done:
		done = True

		if r.match (name):
			g = r.match (name).groups()
			name = g[0]
			done = False

	return name

def list_eq (l1, l2):

	if len(l1) != len(l2):
		return False

	for e in l1:
		if e not in l2:
			return False

	return True

# Convience functions for the config

def config_get_value (section, name):
	return rsutil.globals.config.get_value (section, name)

# Convience functions for the options

def options_get_value (name):
	return getattr (rsutil.globals.options, name)

def main ():
	pass

if __name__ == '__main__':
	main ()

